// -*- C++ -*-
// Copyright (C) 2000 Red Hat, Inc.
#ifndef _INTI_BASE_UTF_STRING_H_
#define _INTI_BASE_UTF_STRING_H_

/*
  Inti is a C++ development platform based on glib, GTK+, and the
  Standard Template Library. This header file is machine-generated
  by a script; do not edit it directly.

  This file is distributed under the GNU Library General Public License.
*/
/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <string>

/*$ doc header
  $short Text processing and strings
  $*/

namespace Inti
{
  typedef int unicode_char;

  
   class ustring : public string
  {
  public:
    
    //// Mirror all string constructors
    
    explicit ustring ()
    {
    }

    ustring (const string & s)
      : string (s)
    {
    }
    
    ustring (const string & s,
             size_type pos,
             size_type n = npos)
      : string (s, pos, n)
    {
    }

    ustring (const value_type * p,
             size_type n)
      : string (p, n)
    {
    }

    ustring (const value_type * p)
      : string (p)
    {
    }

    ustring (size_type n,
             value_type c)
      : string (n, c)
    {
    }

    template <class IterType>
    ustring (IterType first,
             IterType last)
      : string (first, last)
    {
    }
   };
  
#if 0
  // This interface is like std::string but it lacks
  // a) random access and b) a non-const iterator,
  // because neither of those can be implemented efficiently
  // with UTF8.

  // See Stroustrup Chapter 20 for std::string; gcc STL doesn't
  // support the allocator stuff though, so we don't either.

  // Since there's no non-const iterator, you need to do this to
  // replace a char:
  //   ustring::const_iterator i = str.begin ();
  //   ustring::const_iterator next = i;
  //   ++next;
  //   str.replace (i, next, 'a');
  // This is what the implementation of "*i = 'a'" would look like,
  // but the long form makes it clear that a) it's inefficient and
  // b) it can invalidate outstanding iterators 

  // The conversion to std::string is very fast though, since we have
  // a std::string internally.
  
  class ustring
  {
  private:
    string str_;
    
  public:

    // Stroustrup p. 583
    
    typedef unicode_char value_type;
    typedef string::size_type size_type;
    typedef string::difference_type difference_type;
    typedef unicode_char & reference;
    typedef const unicode_char & const_reference;
    typedef unicode_char *pointer;
    typedef const unicode_char *const_pointer;

    static const size_type npos;
    
    class const_iterator
    {
      


    }; // class const_iterator

    class const_reverse_iterator
    {


    }; // class const_reverse_iterator


    // Stroustrup p. 584

    const_iterator begin () const;
    const_iterator end () const;

    const_reverse_iterator rbegin () const;
    const_reverse_iterator rend () const;

    ///////////// Nonstandard stuff not found in std::string
    ///////////// (but not all the nonstandard stuff, we also
    ///////////// have a bunch of wstring/string interop stuff)
    
    string & str () { return str_; }
    const string & str () const { return str_; }

    // get iterator at character offset char_offset (O(n) operation
    // so not provided as operator[])
    const_iterator offset (int char_offset) const;

    ///////////// end nonstandard stuff
    
    //// Mirror all string constructors (Stroustrup p. 585)
    
    ustring ()
    {
    }

    // default copy/assignment/destruct work fine

    
    // This is NOT explicit, for better or worse
    ustring (const string & s)
      : str_(s)
    {
    }
    
    ustring (const string & s,
             string::size_type pos,
             string::size_type n = string::npos)
      : str_ (s, pos, n)
    {
    }

    ustring (const char * p,
             size_type n)
      : str_ (p, n)
    {
    }

    ustring (const char * p)
      : str_ (p)
    {
    }

    
#if 0
    ustring (const ustring & s,
             size_type pos,
             size_type n = npos)
      // nope // : str_ (s, pos, n)
    {
    }
    
    ustring (const value_type * p,
             string::size_type n)
      // nope //      : str_ (p, n)
    {
    }
    
    ustring (size_type n,
             value_type c)
      // nope // : str_ (n, c)
    {
    }

    template <class IterType>
    ustring (IterType first,
             IterType last)
      // nope // : str_ (first, last)
    {
    }

    ustring (const wstring & s);
    ustring (const wstring & s,
             wstring::size_type pos,
             wstring::size_type n = wstring::npos);
#endif

    // Stroustrup p. 588

    // using default assignment from ustring
    
    ustring & operator= (const value_type * src);
    ustring & operator= (const char *src);
    ustring & operator= (unicode_char ch);
    ustring & operator= (const string & src);
    ustring & operator= (const wtring & src);
    

    ustring & assign (const ustring & src);
    ustring & assign (const ustring & src,
                      size_type pos,
                      size_type n);

    ustring & assign (const unicode_char *p, size_type n);
    ustring & assign (const unicode_char *p);
    ustring & assign (const char *p);
    ustring & assign (const char *p, size_type n);
    ustring & assign (size_type n, unicode_char ch);
    template <class IterType>
    ustring & assign (IterType first, IterType last)
    {
      // FIXME
    }

    ustring & assign (const string & str);

    ustring & assign (const string & str,
                      size_type pos,
                      size_type n);

    ustring & assign (const wstring & str);
    
    ustring & assign (const wstring & str,
                      size_type pos,
                      size_type n);

    // Stroustrup p. 589

    const char *c_str () const { return str_.c_str (); }
    const char *data () const { return str_.data (); }

    size_type copy (unicode_char *p, size_type n, size_type pos = 0) const;
    size_type copy (const char *p, size_type n, size_type pos = 0) const;

    // Stroustrup p. 590

    int compare (const ustring & s);
    int compare (const char *p) const;
    int compare (const unicode_char *p) const;
    int compare (const string & s) const;

    int compare (size_type pos, size_type n, const ustring & s) const;
    int compare (size_type pos, size_type n, const ustring & s,
                 size_type pos2, size_type n2) const;
    int compare (size_type pos, size_type n, const unicode_char *p,
                 size_type n2 = npos) const;
    int compare (size_type pos, size_type n, const char *p,
                 size_type n2 = npos) const;
    int compare (size_type pos, size_type n, const string & s) const;
    int compare (size_type pos, size_type n, const string & s,
                 size_type pos2, size_type n2) const;
    

    // Stroustrup p. 592

    ustring & operator+= (const ustring & s);
    ustring & operator+= (const string & s);
    ustring & operator+= (const wstring & s);
    ustring & operator+= (const char *p);
    ustring & operator+= (unicode_char ch);
    void push_back (unicode_char ch);

    ustring & append (const ustring & s);
    ustring & append (const ustring & s, size_type pos, size_type n);
    ustring & append (const string & s);
    ustring & append (const string & s,
                      string::size_type pos,
                      string::size_type n);
    ustring & append (const wstring & s);
    ustring & append (const wstring & s,
                      wstring::size_type pos,
                      wstring::size_type n);

    ustring & append (const char *p, size_type n);
    ustring & append (const char *p);
    ustring & append (size_type n, unicode_char c);
    template <class IterType>
    ustring & append (IterType first, IterType last)
    {
      // FIXME
    }

    // extra-nonstandard appends
    ustring & append (const ustring & s, const_iterator pos, size_type n);

    
    ustring & insert (size_type pos, const ustring & s);
    ustring & insert (size_type pos, const ustring & s,
                      size_type pos2, size_type n);
    ustring & insert (size_type pos, const char *p, size_type n);
    ustring & insert (size_type pos, const char *p);
    ustring & insert (size_type pos, size_type n, unicode_char ch);
    

    const_iterator insert (const_iterator i, unicode_char ch);
    void insert (const_iterator i, size_type n, unicode_char ch);

    template <class IterType>
    void insert (const_iterator i, IterType first, IterType last)
    {
      // FIXME 
    }

    // extra-nonstandard inserts
    void insert (const_iterator i, const char *p);
    void insert (const_iterator i, const string & p);
    void insert (const_iterator i, const wstring & p);

    
    
    
    size_type size () const;
    size_type length () const { return size (); }
    ustring substr (size_type i = 0, size_type n = npos);
    unicode_char & at (size_type i);
    const unicode_char & at (size_type i) const;

    // Stroustrup p. 594 has the find functions - these
    // are not implemented. They return size_type, which
    // would be super-inefficient. Variants returning
    // an iterator are implemented instead.

    
    // NOTE all the find variants are modified to return an iterator
    // instead of an index, and take iterator args. It's just too
    // inefficient otherwise.

    const_iterator find (const ustring & substr) const;
    const_iterator find (const ustring & substr,
                         const_iterator i) const;
    const_iterator find (const ustring & substr,
                         const_iterator i,
                         size_type n) const;

    
    const_iterator find (const char * substr) const;
    const_iterator find (const char * substr,
                         const_iterator i) const;
    const_iterator find (const char * substr,
                         const_iterator i,
                         size_type n) const;


    const_iterator find (const string & substr) const;
    const_iterator find (const string & substr,
                         const_iterator i) const;
    const_iterator find (const string & substr,
                         const_iterator i,
                         size_type n) const;

    
    const_iterator find (unicode_char c) const;
    const_iterator find (unicode_char c, const_iterator i) const;



    const_iterator rfind (const ustring & substr) const;
    const_iterator rfind (const ustring & substr,
                          const_iterator i) const;
    const_iterator rfind (const ustring & substr,
                          const_iterator i,
                          size_type n) const;

    
    const_iterator rfind (const char * substr) const;
    const_iterator rfind (const char * substr,
                          const_iterator i) const;
    const_iterator rfind (const char * substr,
                          const_iterator i,
                          size_type n) const;


    const_iterator rfind (const string & substr) const;
    const_iterator rfind (const string & substr,
                          const_iterator i) const;
    const_iterator rfind (const string & substr,
                          const_iterator i,
                          size_type n) const;
    
    
    const_iterator rfind (unicode_char c) const;
    const_iterator rfind (unicode_char c, const_iterator i) const;

    

    
    const_iterator find_first_of (const ustring & chars, const_iterator i = 0) const;
    const_iterator find_first_of (const char * chars, const_iterator i = 0) const;
    const_iterator find_first_of (const char * chars, const_iterator i, const_iterator n) const;
    const_iterator find_first_of (unicode_char c, const_iterator i = 0) const;

    const_iterator find_last_of (const ustring & chars, const_iterator i = npos) const;
    const_iterator find_last_of (const char * chars, const_iterator i = npos) const;
    const_iterator find_last_of (const char * chars, const_iterator i, const_iterator n) const;
    const_iterator find_last_of (unicode_char c, const_iterator i = npos) const;
    
    const_iterator find_first_not_of (const ustring & chars, const_iterator i = 0) const;
    const_iterator find_first_not_of (const char * chars, const_iterator i = 0) const;
    const_iterator find_first_not_of (const char * chars, const_iterator i, const_iterator n) const;
    const_iterator find_first_not_of (unicode_char c, const_iterator i = 0) const;

    const_iterator find_last_not_of (const ustring & chars, const_iterator i = npos) const;
    const_iterator find_last_not_of (const char * chars, const_iterator i = npos) const;
    const_iterator find_last_not_of (const char * chars, const_iterator i, const_iterator n) const;
    const_iterator find_last_not_of (unicode_char c, const_iterator i = npos) const;
    
  };

  // Stroustrup p. 591

  bool operator== (const ustring & lhs, const ustring & rhs)
  {
    return lhs.str () == rhs.str ();
  }
  
  bool operator== (const char *p, const ustring & rhs)
  {
    return p == rhs.str ();
  }

  bool operator== (const ustring & lhs, const char *p)
  {
    return lhs.str () == p;
  }

  bool operator== (const string & lhs, const ustring & rhs)
  {
    return lhs == rhs.str ();
  }

  bool operator== (const ustring & lhs, const string & rhs)
  {
    return lhs.str () == rhs;
  }

  // !=
  bool operator!= (const ustring & lhs, const ustring & rhs)
  {
    return lhs.str () != rhs.str ();
  }
  
  bool operator!= (const char *p, const ustring & rhs)
  {
    return p != rhs.str ();
  }

  bool operator!= (const ustring & lhs, const char *p)
  {
    return lhs.str () != p;
  }

  bool operator!= (const string & lhs, const ustring & rhs)
  {
    return lhs != rhs.str ();
  }

  bool operator!= (const ustring & lhs, const string & rhs)
  {
    return lhs.str () != rhs;
  }

  // >
  bool operator> (const ustring & lhs, const ustring & rhs)
  {
    return lhs.str () > rhs.str ();
  }
  
  bool operator> (const char *p, const ustring & rhs)
  {
    return p > rhs.str ();
  }

  bool operator> (const ustring & lhs, const char *p)
  {
    return lhs.str () > p;
  }

  bool operator> (const string & lhs, const ustring & rhs)
  {
    return lhs > rhs.str ();
  }

  bool operator> (const ustring & lhs, const string & rhs)
  {
    return lhs.str () > rhs;
  }

  // <
  bool operator< (const ustring & lhs, const ustring & rhs)
  {
    return lhs.str () < rhs.str ();
  }
  
  bool operator< (const char *p, const ustring & rhs)
  {
    return p < rhs.str ();
  }

  bool operator< (const ustring & lhs, const char *p)
  {
    return lhs.str () < p;
  }

  bool operator< (const string & lhs, const ustring & rhs)
  {
    return lhs < rhs.str ();
  }

  bool operator< (const ustring & lhs, const string & rhs)
  {
    return lhs.str () < rhs;
  }

  // >=
  bool operator>= (const ustring & lhs, const ustring & rhs)
  {
    return lhs.str () >= rhs.str ();
  }
  
  bool operator>= (const char *p, const ustring & rhs)
  {
    return p >= rhs.str ();
  }

  bool operator>= (const ustring & lhs, const char *p)
  {
    return lhs.str () >= p;
  }

  bool operator>= (const string & lhs, const ustring & rhs)
  {
    return lhs >= rhs.str ();
  }

  bool operator>= (const ustring & lhs, const string & rhs)
  {
    return lhs.str () >= rhs;
  }

  // <=
  bool operator<= (const ustring & lhs, const ustring & rhs)
  {
    return lhs.str () <= rhs.str ();
  }
  
  bool operator<= (const char *p, const ustring & rhs)
  {
    return p <= rhs.str ();
  }

  bool operator<= (const ustring & lhs, const char *p)
  {
    return lhs.str () <= p;
  }

  bool operator<= (const string & lhs, const ustring & rhs)
  {
    return lhs <= rhs.str ();
  }

  bool operator<= (const ustring & lhs, const string & rhs)
  {
    return lhs.str () <= rhs;
  }

  // Stroustrup p. 593

  ustring operator+ (const ustring & lhs, const ustring & rhs)
  {
    return ustring (ustring.str () + ustring.str ());
  }

  ustring operator+ (const ustring & lhs, const char *p)
  {
    return ustring (ustring.str () + p);
  }

  ustring operator+ (const ustring & lhs, unicode_char ch)
  {
    ustring tmp (lhs);
    tmp += ch;
    return tmp;
  }

  ustring operator+ (const ustring & lhs, const string & s)
  {
    return ustring (lhs.str () + s);
  }


  ustring operator+ (const char *p, const ustring & rhs)
  {
    return ustring (p + rhs.str ());
  }

  ustring operator+ (unicode_char ch, const ustring & rhs)
  {
    ustring tmp (1, ch);
    tmp += rhs;
    return rhs;
  }

  ustring operator+ (const string & s, const ustring & rhs)
  {
    return ustring (s + rhs.str ());
  }
   
#endif  
  
}; // namespace Inti

#endif // _INTI_BASE_UTF_STRING_H_
